/* ****************************************************************************
  This file is part of KBabel

  Copyright (C) 2004 by StanislavVsinovsky
			    <visnovsky@kde.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************************** */

#include "projectwizard.h"
#include "projectwizardwidget.h"

#include "kbprojectmanager.h"

#include <klineedit.h>
#include <klocale.h>
#include <kurlrequester.h>

using namespace KBabel;

ProjectWizard::ProjectWizard(QWidget *parent,const char *name)
        : KDialogBase(parent,name,true
                ,i18n("Caption of dialog","Create a new project") 
                , User1|Cancel, User1)
{
    _wizard=new ProjectWizardWidget(this,"project wizard widget");
   
    connect( this, SIGNAL(user1Clicked()), this, SLOT(check()));
    connect( _wizard->_projectName, SIGNAL(textChanged(const QString &)), this, SLOT(textChanged(const QString &)));
    connect( _wizard->_projectFile, SIGNAL(textChanged(const QString &)), this, SLOT(textChanged(const QString &)));

    enableButton( User1, false);

    setButtonText( User1, i18n("Button label", "&Create") );
    setMainWidget(_wizard);
}

Project::Ptr ProjectWizard::project()
{
    Project::Ptr p = ProjectManager::open( _wizard->_projectFile->url() );
    p->setName( _wizard->_projectName->text() );
    return p;
}

void ProjectWizard::check()
{
    accept();
}

void ProjectWizard::textChanged(const QString &)
{
    enableButton( User1, !_wizard->_projectName->text().isEmpty() && !_wizard->_projectFile->url().isEmpty() );
}

Project::Ptr ProjectWizard::newProject()
{
    ProjectWizard* dialog = new ProjectWizard();
    if( dialog->exec() == QDialog::Accepted )
    {
	Project::Ptr res = dialog->project();
	delete dialog;
	return res;
    }
    
    return 0;
}

#include "projectwizard.moc"
